#include <Arduino.h>
#include "rc522.h"
#include "SPI.h"
#include <MFRC522.h>
#include "rotary_encoder.h"

#include "containers/LedStripRGBW.h"
#include "drivers/Esp32DriverRGBW.h"
#include "effects/Circular.h"
#include "effects/Static.h"
#include "effects/AlexaSwipe.h"
#include "effects/RandomTwoColorInterpolation.h"

#include "Messages.h"

#include "TaskLed.h"

MFRC522 rfid; // Instance of the class

MFRC522::MIFARE_Key key;

//LedStrip led(46, 23);
LedStripRGBW<51> ledStrip;
Esp32DriverRGBW ledDriver;

LedTask<decltype(ledStrip)> ledTask;

bool fox;
void tag_handler(uint8_t *sn)
{
  // serial number is always 5 bytes long
  if (sn != nullptr)
  {

    Serial.printf("Tag: %#x %#x %#x %#x %#x\n",
                  sn[0], sn[1], sn[2], sn[3], sn[4]);
    sendMessageToHost(MsgRfidTokenRead{{sn[0], sn[1], sn[2], sn[3], sn[4]}});
    if (sn[4] == 0x30)
    {
      Serial.println("Fuchs");
      fox = true;
      //ledTask.startEffect(EffectCircularConfig{2 * 360, 180, ColorRGBW{0, 0, 255, 0}});
      ledTask.startEffect(EffectAlexaSwipeConfig{20, 30, 3 * 360, 3, 180, true, ColorRGBW{0, 255, 0, 0}, ColorRGBW{0, 0, 255, 0}});
      delay(1000);
      ledTask.startEffect(EffectRandomTwoColorInterpolationConfig{6000, true, 6, false, false, rgb2hsv(ColorRGBW{128, 0, 0, 0}),
                                                                  rgb2hsv(ColorRGBW{0, 0, 128, 0})});
    }
    if (sn[4] == 0xf0)
    {
      Serial.println("Eule");
      fox = false;
      ledTask.startEffect(EffectCircularConfig{360, 180, ColorRGBW{0, 0, 255, 0}});
    }
    if (sn[4] == 0xe9)
    {
      Serial.println("Elephant");
      fox = true;
      ledTask.startEffect(EffectAlexaSwipeConfig{20, 30, 3 * 360, 3, 180, true, ColorRGBW{0, 0, 255, 0}, ColorRGBW{0, 200, 255, 0}});
      delay(1000);
      ledTask.startEffect(EffectRandomTwoColorInterpolationConfig{6000, true, 3, false, false, rgb2hsv(ColorRGBW{0, 0, 255, 0}),
                                                                  rgb2hsv(ColorRGBW{0, 200, 255, 0})});
    }
  }
  else
  {
    Serial.println("Nichts");
    if (fox)
    {
      fox = false;
      ledTask.startEffect(EffectAlexaSwipeConfig{20, 30, 3 * 360, 3, 180, false, ColorRGBW{0, 255, 0, 0}, ColorRGBW{0, 0, 255, 0}});
    }
    else
      ledTask.startEffect(EffectStaticConfig{ColorRGBW{0, 0, 0, 0}});
  }
  //led.transmit();
}
QueueHandle_t event_queue;

rotary_encoder_info_t info;
void setup()
{
  Serial.begin(115200);

  //led.begin();
  ledDriver.begin(23, 0);

  const rc522_start_args_t start_args = {
      21, // MISO
      5,  // MOSI
      18, // SCK
      19, // SDA
      VSPI_HOST,
      &tag_handler,
      125,      // scan_interval_ms
      8 * 1024, // stacksize
      4         // task priority
  };
  rc522_start(start_args);

  ESP_ERROR_CHECK(gpio_install_isr_service(0));
  ESP_ERROR_CHECK(rotary_encoder_init(&info, GPIO_NUM_26, GPIO_NUM_27));
  ESP_ERROR_CHECK(rotary_encoder_enable_half_steps(&info, false));
  event_queue = rotary_encoder_create_queue();
  ESP_ERROR_CHECK(rotary_encoder_set_queue(&info, event_queue));

  //button leds
  //pinMode(33, OUTPUT);
  //digitalWrite(33, HIGH);
  //pinMode(12, OUTPUT);
  //digitalWrite(12, HIGH);

  //// button in
  //pinMode(25, INPUT_PULLUP);
  //pinMode(14, INPUT_PULLUP);
  //pinMode(13, INPUT_PULLUP);
  ledTask.begin(ledStrip, ledDriver);
  ledTask.startEffect(EffectStaticConfig{ColorRGBW{0, 0, 0, 0}});
}

bool btn2state = true;

void loop()
{
  handleIncomingMessagesFromHost(&ledTask);
}